package jp.go.aist.hrp.simulator;


/**
* jp/go/aist/hrp/simulator/ControllerOperations.java .
* IDL-to-Java コンパイラ (ポータブル), バージョン "3.1" で生成
* 生成元: C:/Users/ishimura/workspaces/ai_project/OpenHRP3Client/src/idl/OpenHRP/Controller.idl
* 2009年2月18日 12時58分36秒 JST
*/


/**
	 * @if jp
	 * @brief Controller インターフェース。
	 *
	 * 制御ソフトの役割を果たします。 Controller は以下の流れを行うように作成してください。
	 *
	 * -# コンストラクタで起動時の初期化
	 * -# start() でシミュレーション開始時の初期化
	 * -# input() でシミュレータからセンサ情報を得る
	 * -# control() センサ情報から制御計算を行う
	 * -# output() でアクチュエータに情報をセットする
	 * 
	 * @note 今後OpenHRPのコントローラはOpenRTMのコンポーネントとして作成するのが標準となります。
	 * 従って、一般的な使用においては本インタフェースを直接使う必要はありません。
	 *
	 * @else
	 * Controller Interface
	 * @endif
	 */
public interface ControllerOperations 
{

  /**
  		 * @if jp
  		 * @brief DynamicsSimulator をセットする。
  		 *
  		 * クライアントが Controller サーバを使用する際、 DynamicsSimulator への参照をセットします。
  		 * @param dynamicssimulator_ DynamicsSimulator への参照
  		 * @endif
  		 */
  void setDynamicsSimulator (jp.go.aist.hrp.simulator.DynamicsSimulator dynamicssimulator_);

  /**
  		 * @if jp
  		 * @brief ViewSimulatorをセットする。
  		 *
  		 * setDynamicsSimulator() と同様に ViewSimulator を使用する Controller に
  		 * 接続する際はこのメソッドでセットします。
  		 *
  		 * @param viewsimulator_ ViewSimulator への参照
  		 * @endif
  		 */
  void setViewSimulator (jp.go.aist.hrp.simulator.ViewSimulator viewsimulator_);

  /**
  		 * @if jp
  		 * @brief シミュレーションループが始まる前に１回呼ばれます。
  		 *
  		 * シミュレーション前の初期化はここに書いてください。
  		 * @else
  		 * This method is called just before beginning periodic calls of control().
  		 *
  		 * Initialization that requires the initial state of a robot
  		 * should be done in this method.
  		 * @endif
  		 */
  void start ();

  /**
  		 * @if jp
  		 * @brief 制御計算を行います。
  		 *
  		 * シミュレーションループにおいて input() で取得した情報を使用して計算を行い、 output() で出力します。
  		 * @endif
  		 */
  void control ();

  /**
  		 * @if jp
  		 * @brief センサ情報の取得を行います。
  		 *
  		 * DynamicsSimulator::getCharacterSensorState() 等を使用してセンサ情報を取得します。
  		 * @endif
  		 */
  void input ();

  /**
  		 * @if jp
  		 * @brief アクチュエータなどへ情報を出力します。
  		 *
  		 * DynamicsSimulator::setCharacterLinkData() 等を使用してシミュレータへ情報を出力します。
  		 * @endif
  		 */
  void output ();

  /**
  		 * @if jp
  		 * シミュレーションループが終了した際に呼ばれます。
  		 * @else
  		 * This method is called when the control loop is finished.
  		 * @endif
  		 */
  void stop ();

  /**
  		 * @if jp
  		 * サーバを終了します。
  		 * @endif
  		 */
  void destroy ();

  /**
  		 * @if jp
  		 * controllerのtimeStepを設定します。
  		 * @endif
  		 */
  void setTimeStep (double timeStep);
} // interface ControllerOperations
